require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  MUTATE_PARAMS = {
    key: 'String',
    label: 'String',
    request_name: 'String',
    code_ikos: 'String'
  }

  ACCEPTABLE_PARAMS = %w[
    key
    label
    request_name
    code_ikos
    created_at
    updated_at
  ]

  RSpec.shared_examples 'admin only' do
    before do
      authenticated_header(request, user)
      subject
    end
    it 'returns unauthorized error' do
      expect(response).to have_http_status 401
    end
  end

  describe Api::Dmo::HousingChannelsController, :type => :api do
    routes { Hermes::Engine.routes }
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    check_mutable_params(
      model: Hermes::HousingChannel, 
      mutate_params: MUTATE_PARAMS, 
      factory: :housing_channel,
      user_roles: ['Administrator'],
      min_params: {
        key: 'test'
      }
    )

    let!(:responsible) { create(:user, roles: ['Administrator']) }
    let!(:roles) { %w[bailleur entreprise demandeur Manager Supervisor] }
    let!(:user) { create(:user, roles: roles.sample(rand(1..3))) }

    let(:channel) { create :housing_channel, key: 'dalo' }
    let(:params) do
      { attributes: { key: 'priority_public', label: 'Public prioritaire' } }
    end

    before { authenticated_header(request, responsible) }

    describe 'GET index' do
      let!(:channels) { create_list :housing_channel, 5 }
      subject { get 'index' }

      it_behaves_like 'admin only'

      it 'returns status code 200' do
        subject
        expect(response).to have_http_status 200
      end

      it 'returns a valid json' do
        subject
        expect(json['data'].length).to eq 5
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq 5
      end
    end

    describe 'GET show' do
      subject { get 'show', { id: channel.id } }

      it_behaves_like 'admin only'

      it 'returns status code 200' do
        subject
        expect(response).to have_http_status 200
      end

      it 'returns a valid json' do
        subject
        expect(json['data']['id']).to eq channel.id.to_s
        expect(json['data']['attributes']['key']).to eq 'dalo'
      end
    end

    describe 'POST create' do
      subject { post 'create', { data: params } }

      it_behaves_like 'admin only'

      it 'succeeds' do
        subject
        expect(response).to have_http_status 201
      end

      it 'creates a housing channel' do
        expect do
          post 'create', { data: params }
        end.to change { Hermes::HousingChannel.count }.by 1
      end

      it 'returns a valid json' do
        subject
        expect(json['data']['attributes']['key']).to eq 'priority_public'
        expect(json['data']['attributes']['label']).to eq 'Public prioritaire'
      end
    end

    describe 'PATCH update' do
      subject { patch 'update', { id: channel.id, data: params } }

      it_behaves_like 'admin only'

      it 'succeeds' do
        subject
        expect(response).to have_http_status 200
      end

      it 'updates the channel' do
        expect do
          subject
        end.to change { channel.reload.key }.from('dalo').to 'priority_public'
      end

      it 'returns a valid json' do
        subject
        expect(json['data']['id']).to eq channel.id.to_s
        expect(json['data']['attributes']['key']).to eq 'priority_public'
        expect(json['data']['attributes']['label']).to eq 'Public prioritaire'
      end
    end

    describe 'DELETE destroy' do
      let(:rand_user) { create :user }
      let!(:owned_channel) { create :housing_channel, user_id: responsible.id }

      subject { delete 'destroy', { id: owned_channel.id.to_s } }

      it_behaves_like 'admin only'

      context 'for owner' do
        it 'succeeds' do
          subject
          expect(response).to have_http_status 204
        end

        it 'destroys the channel' do
          expect do
            subject
          end.to change { Hermes::HousingChannel.count }.by -1
        end
      end

      context 'for another user' do
        before { authenticated_header(request, rand_user) }

        it 'returns error' do
          subject
          expect(response).to have_http_status 401
        end
      end
    end
  end

  require_relative '../../swagger_helper'

  describe Hermes::Api::Dmo::HousingChannelsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    TAGS_API_DMO_HOUSING_CHANNELS = 'API_DMO_HOUSING_CHANNELS'

    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes
    let!(:responsible) { create(:user, roles: ['Administrator']) }

    context 'as admin' do
      let!(:admin) { create(:administrator) }

      path '/api/dmo/housing_channels' do
        post 'Create housing channel' do
          tags TAGS_API_DMO_HOUSING_CHANNELS
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
          json_params[:key] = { type: :string, required: true }
          json_params[:label] = { type: :string }
          json_params[:request_name] = { type: :string }

          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '201', 'housing channel created' do
            let(:Authorization) { "Bearer #{token_for(responsible)}" }
            let(:params) do
              jsonapi_attributes(attributes_for(:housing_channel))
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_channels'
            end
          end
        end
      end

      path '/api/dmo/housing_channels' do
        get 'Retrieves all housing channels' do
          tags TAGS_API_DMO_HOUSING_CHANNELS
          produces 'application/json'

          ACCEPTABLE_PARAMS.each do |acceptable_param|
            type = ''
            if MUTATE_PARAMS.include?(acceptable_param)
              type = nice_type(MUTATE_PARAMS[acceptable_param])
            end
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end

          security [Bearer: {}]
          paginator

          response '200', 'housing channels found' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let!(:housing_channels) { create_list(:housing_channel, 5) }

            include_context 'with integration test'
            include_context 'valid jsonapi collection' do
              let(:expected_collection) { housing_channels }
            end
          end
        end
      end

      path '/api/dmo/housing_channels/{id}' do
        get 'Retrieves a single housing channel by id' do
          tags TAGS_API_DMO_HOUSING_CHANNELS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '200', 'housing channel found' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:housing_channel) { create(:housing_channel) }
            let(:id) { housing_channel.id }

            include_context 'with integration test'
            include_context 'valid jsonapi ressource' do
              let(:expected_ressource) { housing_channel }
            end
          end
        end
      end

      path '/api/dmo/housing_channels/{id}' do
        put 'edit housing channel' do
          tags TAGS_API_DMO_HOUSING_CHANNELS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '200', 'housing channel updated' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:housing_channel) { create(:housing_channel) }
            let(:id) { housing_channel.id }
            let(:params) do
              jsonapi_attributes(
                 { key: 'string' }
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_channels'
            end
          end
        end
      end

      path '/api/dmo/housing_channels/{id}' do
        delete 'Delete a single housing channel by id' do
          tags TAGS_API_DMO_HOUSING_CHANNELS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '204', 'housing channel deleted' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:id) { create(:housing_channel, responsible: admin.id).id }
            run_test!
          end
        end
      end
    end
  end
end