require 'rails_helper'
module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  MUTATE_PARAMS = {
    activation_date: 'Date',
    department: 'String',
    district: 'String',
    threshold_type: 'String',
    priority_public: 'Boolean',
    new_offer: 'Boolean',
    target_by_typologies: 'Hash',
    operation_name:"String",
    lessor_id: ['lessor1', 'lessor2', 'lessor3'],
    agency_id: 'String',
  }
  
  ACCEPTABLE_PARAMS = %w[
    als_id
    activation_date
    department
    district
    threshold_type
    priority_public
    new_offer
    target_by_typologies
    present_typologies
    realized_by_typologies
    objective_difference_by_typologies
    lessor_id
  ]

  describe Api::Dmo::LessorObjectivesController, :type => :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    check_mutable_params(
      model: Hermes::LessorObjective, 
      mutate_params: MUTATE_PARAMS, 
      factory: :lessor_objective,
      other_factory:[
        {factory: :lessor, id: 'lessor1'},
        {factory: :lessor, id: 'lessor2'},
        {factory: :lessor, id: 'lessor3'},
      ],
    )

    let!(:responsible) {create(:user, roles: ['Administrator'])}
    let!(:user) {create(:user)}
    let!(:lessor2) { create(:lessor) }
    let!(:activation_date) {(Time.now + 5.month).to_s}
    let!(:lessor_objective1) do
      create(
          :lessor_objective,
          department: '34',
          district: 'Montpellier',
          threshold_type: 'PLUS 60%',
          #priority_public: true,
          new_offer: false,
          target_by_typologies: { all: 24 },
          lessor_id: lessor2.id.to_s,
          activation_date: activation_date
      )
    end

    #TESTS BASIQUES D'API

    describe 'POST /lessor_objectives' do
      describe 'Create lessor_objective with minimal valid datas' do
        before :each do
          authenticated_header(request, responsible)
          post 'create', {
              data: {
                  attributes: {
                  }
              }
          }
        end
        it 'returns status code 201' do
          expect(response).to have_http_status(201)
        end
      end
      describe 'impossible to create if already exists' do
        before :each do
          authenticated_header(request, responsible)
          post 'create', {
              data: {
                  attributes: {
                      department: '34',
                      district: 'Montpellier',
                      threshold_type: 'PLUS 60%',
                      #priority_public: true,
                      new_offer: false,
                      target_by_typologies: { all: 10 },
                      lessor_id: lessor2.id.to_s,
                      activation_date: activation_date
                  }
              }
          }
        end
        it 'returns status code 422' do
          expect(response).to have_http_status(422)
        end
      end

      #lessor__id: 'Array'
      describe 'post request with lessor_id' do
        let!(:lessor) {create(:lessor)}

        before :each do
          authenticated_header(request, responsible)
          post 'create', {
              data: {
                  attributes: {
                      lessor_id: lessor.id.to_s
                  }
              }
          }
        end
        it 'returns status code 201 and the value appears in json' do
          expect(response).to have_http_status(201)
          expect(json['data']['attributes']['lessor_id']).to eq(lessor.id.to_s)
        end
      end
    end

    describe 'GET /lessor_objectives' do
      describe 'GET /lessor_objectives' do
        let!(:lessor_objective1) {create(:lessor_objective,threshold_type: 'PLUS 6%', user_id: responsible.id.to_s)}
        let!(:lessor_objective3) {create(:lessor_objective,threshold_type: 'PLUS 0%', user_id: responsible.id.to_s)}
        let!(:lessor_objective2) {create(:lessor_objective,threshold_type: 'PLUS60%', user_id: responsible.id.to_s)}
        before :each do
          authenticated_header(request, responsible)
          get 'index'
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(3)
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(3)
        end
      end

      describe 'GET /lessor_objectives with batch of ids' do
        let!(:lessor_objective1) {create(:lessor_objective,threshold_type: 'PLUS 6%', user_id: responsible.id.to_s)}
        let!(:lessor_objective3) {create(:lessor_objective,threshold_type: 'PLUS 0%', user_id: responsible.id.to_s)}
        let!(:lessor_objective2) {create(:lessor_objective,threshold_type: 'PLUS60%', user_id: responsible.id.to_s)}
        before :each do
          authenticated_header(request, responsible)
          get 'index', {ids: [lessor_objective1.id.to_s, lessor_objective2.id.to_s]}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(2)
        end
      end

      ACCEPTABLE_PARAMS.each do |acceptable_param|
        next if ['lessor_id'].include?(acceptable_param)
        type = MUTATE_PARAMS[acceptable_param.to_sym] || MUTATE_PARAMS[acceptable_param.to_s]
        if type
          describe 'GET /lessor_objectives with sort on field ' + acceptable_param + ' works ' do
            let!(:lessor_objective3) {create(:lessor_objective, acceptable_param => get_typed_value(type, 2), user_id: responsible.id.to_s)}
            let!(:lessor_objective1) {create(:lessor_objective, acceptable_param => get_typed_value(type, 0), user_id: responsible.id.to_s)}
            let!(:lessor_objective2) {create(:lessor_objective, acceptable_param => get_typed_value(type, 1), user_id: responsible.id.to_s)}
            before :each do
              authenticated_header(request, responsible)
              get 'index', {sort: {acceptable_param => 1}}
            end
            it 'returns sorted results' do
              expect(response.status).to eq(200)
              if type == 'DateTime'
                lessor_objective_attribute = json['data'].map {|unit| DateTime.parse(unit['attributes'][acceptable_param])}
                expect(lessor_objective_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
              elsif type == 'Date'
                lessor_objective_attribute = json['data'].map {|unit| Date.parse(unit['attributes'][acceptable_param])}
                expect(lessor_objective_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
              else
                lessor_objective_attribute = json['data'].map {|unit| unit['attributes'][acceptable_param]}
                expect(lessor_objective_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
              end
            end
          end
        end
      end
      describe 'GET /lessor_objectives with operators' do
        let!(:lessor_objective1) {create(:lessor_objective, department: 'acceuillant', user_id: responsible.id.to_s)}
        let!(:lessor_objective3) {create(:lessor_objective, department: 'Super', user_id: responsible.id.to_s)}
        let!(:lessor_objective2) {create(:lessor_objective, department: 'bel appart', user_id: responsible.id.to_s)}

        describe 'with $in' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', {department: {'$in': ['acceuillant', 'flamand']}}
          end
          it 'returns results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(1)
          end
        end
        describe 'with $ne' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', {department: {'$ne': 'acceuillant'}}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(2)
          end
        end
        describe 'with $regex' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', {department: {'$regex': /^S/}}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(1)
          end
        end
      end
    end

    describe 'GET /lessor_objectives/:id' do
      describe 'verif it work' do
        let!(:lessor_objective1) {create(:lessor_objective, department: 'acceuillant', user_id: responsible.id.to_s)}
        before :each do
          authenticated_header(request, responsible)
          get 'show', {id: lessor_objective1.id}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
        end
      end
      describe 'verif it work not work with a non authorized user' do
        let!(:lessor_objective1) {create(:lessor_objective, user_id: responsible.id.to_s)}

        before :each do
          authenticated_header(request, user)
          get 'show', {id: lessor_objective1.id}
        end
        it 'returns status code 404' do
          expect(response).to have_http_status(404)
        end
      end
    end

    describe 'DELETE /lessor_objectives/:id' do
      let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}
      let!(:lessor_objective) {create(:lessor_objective, user_id: user.id.to_s)}
      before :each do
        authenticated_header(request, user)

        delete 'destroy', {
            id: lessor_objective.id.to_s
        }
      end
      it 'returns status code 204' do
        expect(response).to have_http_status(204)
      end
    end
    
  end

  require_relative '../../swagger_helper'

  # describe Hermes::Api::Dmo::LessorObjectivesController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  #   TAGS_API_DMO_LESSOR_OBJECTIVES = 'API_DMO_LESSOR_OBJECTIVES'
  #   include JwtAuthHelper
  #   extend HermesSwaggerJsonParams
  #   include SwaggerJsonAttributes
  #   let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
  #   let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}

  #   context 'as admin' do
  #     let!(:admin) {create(:administrator)}

  #     path '/api/dmo/lessor_objectives' do
  #       post 'Create lessor_objective' do
  #         tags TAGS_API_DMO_LESSOR_OBJECTIVES
  #         produces 'application/json'
  #         consumes 'application/json'
  #         security [Bearer: {}]
  #         json_params = {}
  #         json_params[:user_id] = {type: :string}
  #         json_params[:lessor_id] = {type: :string}
  #         json_params[:priority_public] = {type: :boolean}
  #         json_params[:new_offer] = {type: :boolean}
  #         json_params[:agency_id] = {type: :string}
  #         MUTATE_PARAMS.each do |key, val|
  #           json_params[key] = {type: nice_type(val)}
  #         end
  #         parameter name: :params, in: :body, schema: jsonify_params(json_params)

  #         response '201', 'lessor_objective' do
  #           let(:Authorization) {"Bearer #{token_for(responsible)}"}
  #           let(:params) do
  #             jsonapi_attributes(
  #                 user_id: responsible.id.to_s
  #             )
  #           end
  #           include_context 'with integration test'
  #           include_context 'valid jsonapi'
  #           include_context 'not containings errors'

  #           after do |_example|
  #             json = JSON.parse(response.body)

  #             expect(json['data']).to be_a Hash
  #             expect(json['data']['id']).to_not be nil
  #             expect(json['data']['type']).to eq 'lessor_objectives'
  #           end
  #         end
  #       end
  #     end

  #     path '/api/dmo/lessor_objectives' do
  #       get 'Retrieves all lessor_objectives' do
  #         tags TAGS_API_DMO_LESSOR_OBJECTIVES
  #         produces 'application/json'

  #         parameter name: 'ids[]', in: :query, type: :array, items: { type: :string }, collectionFormat: :multi, required: false
  #         ACCEPTABLE_PARAMS.each do |acceptable_param|
  #           type = ''
  #           if MUTATE_PARAMS.include?(acceptable_param)
  #             type = nice_type(MUTATE_PARAMS[acceptable_param])
  #           end
  #           parameter name: acceptable_param.to_s, in: :query, type: type, required: false
  #         end
  #         # VALID_OPERATORS.each do |valid_operator|
  #         #  parameter( name: valid_operator.to_s, in: :query,type: :object, required: false)
  #         # end

  #         security [Bearer: {}]
  #         paginator

  #         response '200', 'lessor_objectives found' do
  #           let(:Authorization) {"Bearer #{token_for(admin)}"}
  #           let!(:lessor_objective1) {create(:lessor_objective,threshold_type: 'PLUS 6%', user_id: responsible.id.to_s)}
  #           let!(:lessor_objective3) {create(:lessor_objective,threshold_type: 'PLUS 0%', user_id: responsible.id.to_s)}
  #           let!(:lessor_objective2) {create(:lessor_objective,threshold_type: 'PLUS60%', user_id: responsible.id.to_s)}

  #           #include_context 'with integration test'
  #           include_context 'valid jsonapi collection' do
            
  #           end
  #         end
  #       end
  #     end

  #     path '/api/dmo/lessor_objectives/{id}' do
  #       get 'Retrieves a single lessor_objective by id' do
  #         tags TAGS_API_DMO_LESSOR_OBJECTIVES
  #         produces 'application/json'
  #         parameter name: :id, in: :path, type: :string
  #         security [Bearer: {}]

  #         response '200', 'lessor_objective found' do
  #           let(:Authorization) {"Bearer #{token_for(admin)}"}
  #           let(:lessor_objective) {create(:lessor_objective)}
  #           let(:id) {lessor_objective.id}

  #           include_context 'with integration test'
  #           include_context 'valid jsonapi ressource' do
  #             let(:expected_ressource) {lessor_objective}
  #           end
  #         end
  #       end
  #     end

  #     path '/api/dmo/lessor_objectives/{id}' do
  #       put 'edit lessor_objective' do
  #         tags TAGS_API_DMO_LESSOR_OBJECTIVES
  #         produces 'application/json'
  #         consumes 'application/json'
  #         parameter name: :id, in: :path, type: :string
  #         security [Bearer: {}]
  #         json_params = {}
  #         json_params[:lessor_id] = {type: :string}
  #         json_params[:priority_public] = {type: :boolean}
  #         json_params[:new_offer] = {type: :boolean}
  #         json_params[:agency_id] = {type: :string}
  #         MUTATE_PARAMS.each do |key, val|
  #           json_params[key] = {type: nice_type(val)}
  #         end
  #         parameter name: :params, in: :body, schema: jsonify_params(json_params)

  #         response '200', 'update lessor_objective' do
  #           let(:Authorization) {"Bearer #{token_for(responsible)}"}
  #           let(:lessor_objective) {create(:lessor_objective, user_id: responsible.id.to_s)}
  #           let(:id) {lessor_objective.id}
  #           let(:params) do
  #             jsonapi_attributes(
  #                 id: id,
  #                 user_id: responsible.id.to_s
  #             )
  #           end
  #           include_context 'with integration test'
  #           include_context 'valid jsonapi'
  #           include_context 'not containings errors'

  #           after do |_example|
  #             json = JSON.parse(response.body)

  #             expect(json['data']).to be_a Hash
  #             expect(json['data']['id']).to_not be nil
  #             expect(json['data']['type']).to eq 'lessor_objectives'
  #           end
  #         end
  #       end
  #     end

  #     path '/api/dmo/lessor_objectives/{id}' do
  #       delete 'Delete a single lessor_objective by id' do
  #         tags TAGS_API_DMO_LESSOR_OBJECTIVES
  #         produces 'application/json'
  #         parameter name: :id, in: :path, type: :string
  #         security [Bearer: {}]

  #         response '204', 'lessor_objective deleted' do
  #           let(:Authorization) {"Bearer #{token_for(admin)}"}
  #           let(:id) {create(:lessor_objective, responsible: admin.id).id}
  #           run_test!
  #         end
  #       end
  #     end
  #   end
  # end
end
