require 'rails_helper'
require 'vcr_config'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')
  require_relative '../../swagger_helper'

  ACCEPTABLE_PARAMS = %w[
    first_name
    last_name
    email
    phone_number
  ]

  MUTATE_PARAMS = {
    first_name: 'String',
    last_name: 'String',
    email: ['email2@test.com', 'email3@test.com'],
    phone_number: 'String',
    gender: 'String',
    phone_number: 'String',
    auth_roles: 'Array',
  }

  VALID_OPERATORS = %w[
    $in
    $nin
    $ne
    $gt
    $gte
    $lt
    $lte
    $regex
  ]

  HERMES_CONFIG['identity_provider_key'] = 'als_hermes_salarie'

  describe Api::Dmo::UsersController, :type => :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:identity_provider) {create(:identity_provider,
      key: 'als_hermes_salarie',
      identifier_rights_url: 'https://rec.client.almerys.com/authz/realms/ALS/protocol/openid-connect/token',
      identifier_client_id: 'ALS_HERMES_BACK',
      identifier_client_secret: '77ff390e-6cca-4f09-9840-0e799a47a0ac',
      identifier_subject_issuer: 'ALS',
      account_service_url: 'https://rec.client.almerys.com/auth/realms/ALS/protocol/openid-connect/token',
      account_service_client_id: 'ALS_HERMES_SERVICE_ACCOUNT',
      account_service_client_secret: '28733e0d-c363-44a2-9f85-ea7d4784a4ad',
      create_user_url: 'https://api-rcc.be-ys.com/als-back/rcc/v1/accounts',
      add_roles_user_url: 'https://api-rcc.almerys.com/iam/api/authorization/clients/ALS_HERMES_BACK/users/{login}/roles/{role}?realm=ALS',
      create_user_headers: {
        'Content-Type': 'application/json',
        'X-GeXRT-API-Key': '169c24b0-a763-40f7-877d-7f94967ffea4',
      }
    )}

    let!(:entity1) {create(:entity, siret: 'Super000000001', title: 'entity1')}
    let!(:entity2) {create(:entity, siret: 'Super000000002', title: 'entity2')}
    let!(:entity3) {create(:entity, siret: 'Super000000003', title: 'entity3')}
    let!(:entity4) {create(:entity, siret: 'Super000000003', title: 'entity4')}
    let!(:admin) { create(:user, roles: ['Administrator'], group_ids: [
      entity1.manager_group_id,
      entity2.manager_group_id,
      entity3.manager_group_id
    ]) }
    let!(:user) { create(:user) }
    let!(:superadministrateur) { create(:user, auth_roles: ['superadministrateur']) }


    describe 'GET /users/:id' do
      let!(:user1) { create(:user, email: 'johndoe@mail.com', auth_roles: ['demandeur'])}
      let!(:user2) { create(:user, email: 'johndoe2@mail.com', auth_roles: ['demandeur'])}
      let!(:no_acl_user) { create(:user, email: 'no_acl_user@mail.com', auth_roles: ['bailleur'])}

      it 'not foudn' do
        authenticated_header(request, user1)
        get 'show', { id: "dsqdq"  }
        expect(response).to have_http_status(404)
      end

      it 'as same user' do
        authenticated_header(request, user1)
        get 'show', { id: user1.id.to_s  }
        expect(response).to have_http_status(200)
      end

      it 'id me' do
        authenticated_header(request, user1)
        get 'show', { id: 'me'  }
        expect(response).to have_http_status(200)
        expect(json['data']['id']).to eq(user1.id.to_s)
      end


      it 'as other user' do
        authenticated_header(request, user1)
        get 'show', { id: user2.id.to_s  }
        expect(response).to have_http_status(401)
      end

      it 'as no acl user ' do
        authenticated_header(request, no_acl_user)
        get 'show', { id: user2.id.to_s  }
        expect(response).to have_http_status(401)
      end

    end

    describe 'GET /users' do
      let!(:responsible) { create(:user, auth_roles: ['superadministrateur'])}
      let!(:user1) { create(:user, auth_roles: ['collaborateur offre logement'])}
      let!(:user2) { create(:user, email: 'johndoe2@mail.com')}
      let!(:target_not_allowed) { create :user }
      
      describe 'as collaborateur' do
        before :each do
          authenticated_header(request, responsible)
          get 'index', {email: user2.email}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
      end

      describe 'as collaborateur' do
        before :each do
          authenticated_header(request, user1)
          get 'index', {email: user2.email}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(401)
        end
      end
      describe 'as user' do
        before :each do
          authenticated_header(request, target_not_allowed)
          get 'index', {email: user2.email}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(401)
        end
      end
    end

    describe 'POST /switch' do
      let!(:reader) { create :user, roles: ['Superadmin', 'Administrator'] }
      let!(:target_allowed) { create :user, auth_roles: ['demandeur', 'bailleur', 'entreprise'].sample(1) }
      let!(:target_not_allowed) { create :user }

      it 'returns unauthorized error' do 
        authenticated_header(request, reader)
        post 'switch', { id: target_allowed.id }

        expect(response).to have_http_status 401
        expect(json['message']).to eq 'Vous n\'avez pas les droits nécessaire pour effectuer cette action.'
      end

      shared_examples 'switch applicant' do
        let!(:applicant_1) { create :applicant, auto_set_user: false }
        let!(:demandeur_1) { create :user, auth_roles: ['demandeur'], main_applicant_id: applicant_1.id }
        let!(:demandeur_2) { create :user, auth_roles: ['demandeur'] }
        let!(:applicant_2) { create :applicant, user_id: demandeur_2.id }
        let!(:unkown_applicant) { create :applicant, auto_set_user: false }

        before do
          authenticated_header(request, switcher)
        end
        it 'succeeds for applicant_1' do
          post 'switch', { id: applicant_1.id }
          expect(response).to have_http_status 200
          expect(json["email"]).to eq demandeur_1.email
          expect(json["jwt_token"]).not_to be_empty
        end
        it 'succeeds for applicant_2' do
          post 'switch', { id: applicant_2.id }
          expect(response).to have_http_status 200
          expect(json["email"]).to eq demandeur_2.email
          expect(json["jwt_token"]).not_to be_empty
        end
        it 'returns not found error' do
          post 'switch', { id: unkown_applicant.id }
          expect(response).to have_http_status 404
          expect(json["message"]).to eq 'Utilisateur target non trouvé.'
        end
      end

      shared_examples 'switch allowed targets' do
        context 'when target is not allowed' do
          before do
            authenticated_header(request, switcher)
            post 'switch', { id: target_not_allowed.id }
          end

          it 'returns unauthorized error' do
            expect(response).to have_http_status 422
            expect(json['message']).to eq 'L\'utilisateur target doit être demandeur, bailleur ou entreprise.'
          end
        end
        context 'when target is allowed' do
          before do
            authenticated_header(request, switcher)
            post 'switch', { id: target_allowed.id }
          end
          it 'succeeds' do
            expect(response).to have_http_status 200
          end
          it 'returns target infos' do
            expect(json["email"]).to eq target_allowed.email
            expect(json["jwt_token"]).not_to be_empty
          end
        end

        it_behaves_like 'switch applicant'
      end

      shared_examples 'switch all targets' do
        let(:target) { [target_allowed, target_not_allowed].sample }
        before do
          authenticated_header(request, switcher)
          post 'switch', { id: target.id }
        end
        it 'succeeds' do
          expect(response).to have_http_status 200
        end
        it 'returns target infos' do
          expect(json["email"]).to eq target.email
          expect(json["jwt_token"]).not_to be_empty
        end

        it_behaves_like 'switch applicant'
      end

      context 'as superadministrateur' do
        it_behaves_like 'switch all targets' do
          let(:switcher) { create :user, auth_roles: ['superadministrateur'] }
        end
      end

      context 'as cre' do
        it_behaves_like 'switch all targets' do
          let(:switcher) { create :user, auth_roles: ['cre'] }
        end
      end

      context 'as collaborator' do
        it_behaves_like 'switch allowed targets' do
          let(:switcher) { create :user, auth_roles: ['collaborateur offre logement'] }
        end
      end
    end
  end
end
